<?php

namespace Drp\DynamicAttributes;

use Drp\DynamicAttributes\Attribute;
use Illuminate\Database\Eloquent\Model;

class AttributeSchema extends Model
{
    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'model_attributes_schema';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'field', 'type', 'allow_null', 'model',
    ];

    /**
     * Attribute relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function attributes()
    {
        return $this->hasMany(Attribute::class, 'schema_id', 'id');
    }

    /**
     * Creates a new schema and attribute model for the given values.
     *
     * @param Model   $object     A eloquent model
     * @param string  $field      Field name of the new schema
     * @param mixed   $value      Initial value of the new schema. Will be used
     *                            to infer the type of the schema
     * @param boolean $allowsNull Default: false
     *
     * @return array
     */
    public static function createNewSchemaAndValue(
        Model $object,
        $field,
        $value,
        $allowsNull = true
    ) {
        $schema    = new static();
        $type      = gettype($value);
        $className = get_class($object);

        $data = [
            'field'      => $field,
            'type'       => $type,
            'allow_null' => $allowsNull,
            'model'      => $className,
        ];

        $schema->fill($data)->save();
        $attribute = new Attribute();
        $attribute->fill(
            [
            'value'     => $value,
            'schema_id' => $schema->id,
            'model_id'  => $object->getKey(),
            ]
        );
        $attribute->save();
        $attribute->schema = $schema;

        return [
            'schema'    => $schema,
            'attribute' => $attribute,
        ];
    }
}
